@@ -112,6 +112,7 @@ use Crypt::Perl::PKCS10::Attributes ();
 use Crypt::Perl::PKCS10::ASN1 ();
 use Crypt::Perl::PKCS10::Attributes ();
 use Crypt::Perl::X509::Name ();
+use Crypt::Perl::X ();
 
 use parent qw( Crypt::Perl::ASN1::Encodee );
 
@@ -148,7 +149,7 @@ use constant asn1_macro => 'CertificationRequest';
 sub new {
     my ($class, %opts) = @_;
 
-    my ($key, $attrs, $subject, $sigalg) = @opts{'key', 'attributes', 'subject', 'signature_algorithm'};
+    my ($key, $attrs, $subject) = @opts{'key', 'attributes', 'subject'};
 
     $subject = Crypt::Perl::X509::Name->new( @$subject );
     $attrs = Crypt::Perl::PKCS10::Attributes->new( @$attrs );
@@ -157,7 +158,6 @@ sub new {
         _key => $key,
         _subject => $subject,
         _attributes => $attrs,
-        _sigalg => $sigalg,
     };
 
     return bless $self, $class;
@@ -176,7 +176,7 @@ sub _encode_params {
         my $bits = $key->max_sign_bits();
 
         if ($bits < 224) {
-            die "This key is too weak ($bits bits) to make a secure PKCS #10 CSR.";
+            die Crypt::Perl::X::create('Generic', "This key is too weak ($bits bits) to make a secure PKCS #10 CSR.");
         }
         elsif ($bits < 256) {
             $bits = 224;
@@ -209,11 +209,11 @@ sub _encode_params {
         $pk_der = $key->to_subject_public_der();
     }
     else {
-        die "Key ($key) is not a recognized private key class instance!";
+        die Crypt::Perl::X::create('Generic', "Key ($key) is not a recognized private key class instance!");
     }
 
     $sig_alg = $Crypt::Perl::PKCS10::ASN1::OID{$sig_alg} || do {
-        die "Unrecognized signature algorithm OID: “$sig_alg”";
+        die Crypt::Perl::X::create('Generic', "Unrecognized signature algorithm OID: “$sig_alg”");
     };
 
     my $asn1_reqinfo = Crypt::Perl::ASN1->new()->prepare( $self->ASN1() );
@@ -228,6 +228,9 @@ sub _encode_params {
     #This is a detail germane to the PKCS10 structure, not to the
     #Attributes itself (right??), so it makes sense to do the change here
     #rather than to put “[0] SET” into the ASN1 template for Attributes.
+    #
+    #“use bytes” is not necessary because we know the first character is
+    #0x31, which came from Convert::ASN1.
     substr($attr_enc, 0, 1) = chr 0xa0;
 
     my %reqinfo = (
